import re

from django.db.models.signals import pre_save
from django.dispatch import receiver
from django.utils.html import linebreaks
from django.utils.text import Truncator
from wagtail.images import get_image_model_string

from .services import parse_iptc

IMAGE_MODEL = get_image_model_string()


@receiver(pre_save, sender=IMAGE_MODEL)
def parse_image_meta(sender, **kwargs):
    """
    Parses image meta data from a tif or jpeg and populates the corresponding
    fields of the image model.
    """
    instance = kwargs["instance"]

    # If we have an ID, don't parse again.
    if instance.id is not None:
        return

    meta_dict = parse_iptc(instance.file)

    # Add the meta data to the fields.
    if title := meta_dict.get("headline", ""):
        trimmed_title = Truncator(title.strip()).chars(255)
        instance.title = trimmed_title
        instance.alt = trimmed_title

    if credit := meta_dict.get("credit", ""):
        trimmed_credit = Truncator(credit.strip()).chars(255)
        instance.credit = trimmed_credit

    if caption := meta_dict.get("caption", ""):
        # Wrap plain-text in <p> tags for RichTextField values.
        starts_with_tag = bool(re.search("^<[p|div].*?>", caption))

        if starts_with_tag:
            instance.caption = caption.strip()
        else:
            instance.caption = linebreaks(caption.strip())

    if byline := meta_dict.get("byline", ""):
        trimmed_byline = Truncator(byline.strip()).chars(255)
        instance.byline = trimmed_byline

    if instructions := meta_dict.get("instructions", ""):
        trimmed_instructions = Truncator(instructions.strip()).chars(255)
        instance.usage_terms = trimmed_instructions.strip()

    if copyright_notice := meta_dict.get("copyright_notice", ""):
        trimmed_copyright = Truncator(copyright_notice.strip()).chars(255)
        instance.copyright_notice = trimmed_copyright

    instance.iptc_data = meta_dict



# Generated by Django 4.2 on 2023-11-21 11:34

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):
    dependencies = [
        ("wagtailimagecaptions", "0004_auto_20231121_1134"),
    ]

    operations = [
        migrations.AlterField(
            model_name="captionedimage",
            name="uuid",
            field=models.UUIDField(db_index=True, default=uuid.uuid4, editable=False),
        ),
    ]



# Generated by Django 4.2 on 2023-11-21 11:34

from django.db import migrations
import uuid


def gen_uuid(apps, schema_editor):
    CaptionedImage = apps.get_model("wagtailimagecaptions", "CaptionedImage")

    for row in CaptionedImage.objects.all():
        row.uuid = uuid.uuid4()
        row.save(update_fields=["uuid"])


class Migration(migrations.Migration):
    dependencies = [
        ("wagtailimagecaptions", "0003_captionedimage_byline_and_more"),
    ]

    operations = [
        migrations.RunPython(gen_uuid, reverse_code=migrations.RunPython.noop),
    ]



from django.conf.urls.static import static
from django.contrib import admin
from django.contrib.staticfiles.urls import staticfiles_urlpatterns
from django.urls import include, path
from django.conf import settings

from wagtail import urls as wagtail_urls
from wagtail.admin import urls as wagtailadmin_urls

urlpatterns = [
    path("admin/", admin.site.urls),
    path("wagtail/", include(wagtailadmin_urls)),
    path("", include(wagtail_urls)),
]

urlpatterns += staticfiles_urlpatterns()
urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



import uuid

from django.db import models
from wagtail.fields import RichTextField
from wagtail.images.models import AbstractImage, AbstractRendition, Image
from wagtail.search import index


class CaptionedImage(AbstractImage):
    uuid = models.UUIDField(db_index=True, default=uuid.uuid4, editable=False)
    alt = models.CharField(
        max_length=255,
        blank=True,
        help_text="Set the image alt text used for accessibility.",
    )
    caption = RichTextField(features=["bold", "italic"], null=True, blank=True, help_text="Set the image caption.")
    credit = models.CharField(
        max_length=255,
        blank=True,
        help_text="The name of the provider (e.g. AP, Getty, Reuters).",
    )
    byline = models.CharField(
        max_length=255,
        blank=True,
        help_text="The name(s) of the creator(s) of the image.",
    )
    usage_terms = models.CharField(
        max_length=255,
        blank=True,
        help_text="Rights information and usage limitations associated with the image, including any special restrictions or instructions.",
    )
    copyright_notice = models.CharField(
        max_length=255,
        blank=True,
        help_text="Any necessary copyright notice(s).",
    )
    iptc_data = models.JSONField(null=True, blank=True)

    admin_form_fields = Image.admin_form_fields + (
        "credit",
        "byline",
        "alt",
        "caption",
        "usage_terms",
        "copyright_notice",
    )

    search_fields = AbstractImage.search_fields + [
        index.SearchField("uuid"),
        index.SearchField("caption"),
    ]

    def default_alt_text(self):
        """Return our stored alt value, otherwise Wagtail defaults to the title."""
        if self.alt:
            return self.alt
        return super().default_alt_text


class CaptionedRendition(AbstractRendition):
    image = models.ForeignKey(CaptionedImage, on_delete=models.CASCADE, related_name="renditions")

    class Meta:
        unique_together = (("image", "filter_spec", "focal_point_key"),)
